#!/bin/sh

# use nice to prevent data loss
CMD_NICER ()
{
	echo "$@"
	nice -n -20 $@
}

CMD_LOGGER ()
{
	echo "$@"
	$@
}

SYNC_CONIG ()
{
	#tmp -> pref, misc
	if [ -e /tmp/bt.conf ]; then
		echo "==> Load bt.conf from /tmp ..."
		btconf=`cat /tmp/bt.conf | sed -e 's/\r$//'`
		echo "${btconf}" > /pref/bt.conf
		btconf=`cat /pref/bt.conf | sed -e 's/$/\r/'`
		echo "${btconf}" > /tmp/fuse_d/MISC/bt.conf
	#misc -> pref
	elif [ -e /tmp/fuse_d/MISC/bt.conf ]; then
		echo "==> Load bt.conf from SD/MISC..."
		btconf=`cat /tmp/fuse_d/MISC/bt.conf | sed -e 's/\r$//'`
		echo "${btconf}" > /pref/bt.conf
	#pref -> misc
	elif [ -e /pref/bt.conf ]; then
		mkdir -p /tmp/fuse_d/MISC
		btconf=`cat /pref/bt.conf | sed -e 's/$/\r/'`
		echo "${btconf}" > /tmp/fuse_d/MISC/bt.conf
	#fw -> pref, misc
	else
		cp /usr/local/share/script/bt.conf /pref/bt.conf
		mkdir -p /tmp/fuse_d/MISC
		btconf=`cat /pref/bt.conf | sed -e 's/$/\r/'`
		echo "${btconf}" > /tmp/fuse_d/MISC/bt.conf
	fi
}

reset_conf ()
{
	echo "reset bt.conf"
	cp /usr/local/share/script/bt.conf /pref/bt.conf
	btconf=`cat /pref/bt.conf | sed -e 's/$/\r/'`
	echo "${btconf}" > /tmp/fuse_d/MISC/bt.conf
}

reset_bluez ()
{
	killall bluetoothd 2>/dev/null
	echo "reset bluetoothd"
	rm -f /pref/bluetooth/* /tmp/fuse_d/MISC/bluetooth/*
	if [ "${GATT_PERIPHERAL}" == "YES" ]; then
		CMD_LOGGER bluetoothd -a
	else
		CMD_LOGGER bluetoothd
	fi
}

wait_hci0 ()
{
	n=0
	while [ ! -e /sys/class/bluetooth/hci0/address ] && [ $n -ne 30 ]; do
		n=$(($n + 1))
		CMD_LOGGER sleep 0.1
	done
	if [ ! -e /sys/class/bluetooth/hci0/address ]; then
		echo "There is no BT interface!"
		#exit 1
	fi

	n=0
	wifi1_mac=`hciconfig -a | grep "BD Addr"|awk '{print $3}'`
	while [ "${wifi1_mac}" == "" ] && [ $n -ne 30 ]; do
		n=$(($n + 1))
		CMD_LOGGER sleep 0.1
		wifi1_mac=`hciconfig -a | grep "BD Addr"|awk '{print $3}'`
	done

	n=0
	while [ "${wifi1_mac}" == "00:00:00:00:00:00" ] && [ $n -ne 30 ]; do
		n=$(($n + 1))
		CMD_LOGGER sleep 0.1
		wifi1_mac=`hciconfig -a | grep "BD Addr"|awk '{print $3}'`
	done

	if [ "${wifi1_mac}" == "" ] || [ "${wifi1_mac}" == "00:00:00:00:00:00" ]; then
		echo "There is no BT interface!"
		reset_conf
		exit 1
	fi
}

HCI0_BRINGUP ()
{
	UART_NODE=`echo "${bt_conf}" | grep UART_NODE | cut -c 11-`
	if [ "${UART_NODE}" == "" ]; then
		reset_conf
		return 1
	fi

	UART_BUSY=`grep ${UART_NODE##/*/} /etc/inittab`
	if [ "${UART_BUSY}" != "" ]; then
		echo "Wrong bluetooth UART config in /etc/inittab: ${UART_BUSY}"
		exit 1
	fi

	if [ -e /proc/ambarella/uart1_rcvr ]; then
		#echo "set UART1 to 1 byte FIFO threshold"
		echo 0 > /proc/ambarella/uart1_rcvr
	fi

	BT_EN_GPIO=`echo "${bt_conf}" | grep BT_EN_GPIO | cut -c 12-`
	if [ "${BT_EN_GPIO}" != "" ]; then
		/usr/local/share/script/t_gpio.sh ${BT_EN_GPIO} 0
		/usr/local/share/script/t_gpio.sh ${BT_EN_GPIO} 1
		CMD_LOGGER sleep 0.1
	fi
	UART_BAUD=`echo "${bt_conf}" | grep UART_BAUD | cut -c 11-`
	if [ "${UART_BAUD}" == "" ]; then
		UART_BAUD=750000
	fi

	HCI_DRIVER=`echo "${bt_conf}" | grep HCI_DRIVER | cut -c 12-`
	if [ "${HCI_DRIVER}" != "brcm" ]; then
		if [ -e /tmp/wifi1_mac ]; then
			CMD_NICER hciattach ${UART_NODE} ${HCI_DRIVER} ${UART_BAUD} flow sleep -d -m /tmp/wifi1_mac
		else
			CMD_NICER hciattach ${UART_NODE} ${HCI_DRIVER} ${UART_BAUD} flow sleep -d
		fi
	else
		if [ -e /tmp/wifi1_mac ]; then
			CMD_LOGGER start-stop-daemon -S -b -N -20 --exec brcm_patchram_plus -- --enable_hci --baudrate ${UART_BAUD} --use_baudrate_for_download --patchram /usr/local/bcmdhd/bt.hcd --no2bytes --enable_lpm ${UART_NODE} --bd_addr `cat /tmp/wifi1_mac`
		else
			CMD_LOGGER start-stop-daemon -S -b -N -20 --exec brcm_patchram_plus -- --enable_hci --baudrate ${UART_BAUD} --use_baudrate_for_download --patchram /usr/local/bcmdhd/bt.hcd --no2bytes --enable_lpm ${UART_NODE}
		fi

		if [ "${1}" == "start" ]; then
			wifi_start=`grep wifi_start.sh /etc/init.d/S52wifi`
			if [ -e /tmp/fuse_d/MISC/wifi.conf ]; then
				WIFI_EN_GPIO=`cat /tmp/fuse_d/MISC/wifi.conf | grep -Ev "^#" | grep WIFI_EN_GPIO | cut -c 14-| sed -e 's/\r$//'`
			else
				WIFI_EN_GPIO=`cat /pref/wifi.conf | grep -Ev "^#" | grep WIFI_EN_GPIO | cut -c 14-`
			fi
			if [ "${wifi_start}" == "" ] && [ "${WIFI_EN_GPIO}" != "" ]; then
				echo 0 > /sys/module/ambarella_config/parameters/sd1_slot0_fixed_cd
				/usr/local/share/script/t_gpio.sh ${WIFI_EN_GPIO} 0
			fi
		fi
		CMD_LOGGER sleep 1
		#enable bluesleep dynamic power saving
		echo 1 > /proc/bluetooth/sleep/proto
	fi

	wait_hci0

	#save /tmp/lib/bluetooth
	mounted=`cat /proc/mounts | grep /tmp/lib/bluetooth`
	if [ "${mounted}" == "" ]; then
		mkdir -p /tmp/lib/bluetooth/${wifi1_mac}
		touch /tmp/fuse_a/.exist
		if [ -e /tmp/fuse_a/.exist ]; then
			rm -f /tmp/fuse_a/.exist
			mkdir -p /pref/bluetooth
			echo "mount --bind /pref/bluetooth/ /tmp/lib/bluetooth/${wifi1_mac}"
			mount --bind /pref/bluetooth/ /tmp/lib/bluetooth/${wifi1_mac}
		else
			mkdir -p /tmp/fuse_d/MISC/bluetooth
			echo "mount --bind /tmp/fuse_d/MISC/bluetooth/ /tmp/lib/bluetooth/${wifi1_mac}"
			mount --bind /tmp/fuse_d/MISC/bluetooth/ /tmp/lib/bluetooth/${wifi1_mac}
		fi
	fi

	if [ "${GATT_PERIPHERAL}" == "YES" ]; then
		CMD_LOGGER bluetoothd -a
	else
		CMD_LOGGER bluetoothd
	fi
	CMD_NICER hciconfig hci0 up
}

DO_BT_AGENT ()
{
	if [ "${BT_LEGACY_PIN}" == "" ]; then
		CMD_LOGGER bt-agent -d --auto_pk -s NoInputNoOutput
	else
		echo -e "\033[032m Bluetooth PIN code=${BT_LEGACY_PIN} \033[0m"
		CMD_LOGGER bt-agent -d --auto_pk -p ${BT_LEGACY_PIN}
	fi
	CMD_LOGGER bt-monitor_headset -s /usr/local/share/script/bt_speakergain.sh -d
}

SET_PISCAN ()
{
	if [ "${ISCAN}" == "YES" ] && [ "${PSCAN}" == "YES" ]; then
		CMD_NICER hciconfig hci0 piscan
	elif [ "${ISCAN}" == "YES" ]; then
		CMD_NICER hciconfig hci0 iscan
	elif [ "${PSCAN}" == "YES" ]; then
		CMD_NICER hciconfig hci0 pscan
	else
		CMD_NICER hciconfig hci0 noscan
	fi
	if [ "${BT_DISCOVERABLE_TIMEOUT}" != "0" ]; then
		CMD_LOGGER bt-adapter --set DiscoverableTimeout ${BT_DISCOVERABLE_TIMEOUT}
	fi
	if [ "${BT_PAIRABLE_TIMEOUT}" != "0" ]; then
		CMD_LOGGER bt-adapter --set PairableTimeout ${BT_PAIRABLE_TIMEOUT}
	fi
}

##### main ##########################################

if [ "${1}" == "fast" ]; then
	hciconfig hci0 up
	exit 0
fi

SYNC_CONIG

bt_conf=`cat /pref/bt.conf | grep -Ev "^#"`
DEVICE_NAME=`echo "${bt_conf}" | grep DEVICE_NAME | cut -c 13-`
GATT_PERIPHERAL=`echo "${bt_conf}" | grep GATT_PERIPHERAL | cut -c 17-| tr '[:lower:]' '[:upper:]'`
ISCAN=`echo "${bt_conf}" | grep ISCAN | cut -c 7- | tr '[:lower:]' '[:upper:]'`
PSCAN=`echo "${bt_conf}" | grep PSCAN | cut -c 7- | tr '[:lower:]' '[:upper:]'`
BT_LEGACY_PIN=`echo "${bt_conf}" | grep BT_LEGACY_PIN | cut -c 15-`
BT_DISCOVERABLE_TIMEOUT=`echo "${bt_conf}" | grep BT_DISCOVERABLE_TIMEOUT | cut -c 25-`
BT_PAIRABLE_TIMEOUT=`echo "${bt_conf}" | grep BT_PAIRABLE_TIMEOUT | cut -c 21-`

if [ "${DEVICE_NAME}" != "" ]; then
	sed -i -e 's|^Name =\(.*\)|Name = '${DEVICE_NAME}'|g' /tmp/bluetooth/main.conf
	if [ -e /tmp/fuse_d/MISC/bluetooth/config ]; then
		btconfig=`cat /tmp/fuse_d/MISC/bluetooth/config | sed -e 's|^name \(.*\)|name '${DEVICE_NAME}'|g'`
		echo "${btconfig}" > /tmp/fuse_d/MISC/bluetooth/config
	fi
	if [ -e /pref/bluetooth/config ]; then
		btconfig=`cat /pref/bluetooth/config | sed -e 's|^name \(.*\)|name '${DEVICE_NAME}'|g'`
		echo "${btconfig}" > /pref/bluetooth/config
	fi
fi

if [ ! -e /sys/class/bluetooth/hci0 ]; then
	HCI0_BRINGUP ${@}
else
	if [ "${GATT_PERIPHERAL}" == "YES" ]; then
		CMD_LOGGER bluetoothd -a
	else
		CMD_LOGGER bluetoothd
	fi
fi


SET_PISCAN

DO_BT_AGENT

#in case bluetoothd is dead
bluez=`ps|grep -v grep|grep bluetoothd`
if [ "${bluez}" == "" ]; then
	reset_bluez
	DO_BT_AGENT
fi

if [ "${BT_LEGACY_PIN}" == "" ]; then
	CMD_NICER hciconfig hci0 sspmode 1
else
	CMD_NICER hciconfig hci0 sspmode 0
fi
